;*****************************************************************************************
;  Double Vision
;
;  The Color LCD screen isn't progressive. It won't update each line in sequence.
;  It does odds then evens (or vice-versa) (You can tell when you shut it off, half the
;                                           lines are gone and the rest fade out)
;
;  So what can we do with that?!   We could create two logos from one by changing it's
;  offset radically at each Vertical Blank. Half the lines will be gone, the others will 
;  still be there. You get two images from one logo, and zero flicker (try THAT on an emulator!)
;  
;  This nifty little thing shows you how to create raster colors without any interrupts, as
;  well as creating two blending logos from one actual graphic by using the Double Vision
;  technique :)
;  BTW: The raster bars go in front and then behind the logo as it goes around   
;
;  All Code by -Pan- of Anthrox  (C) 1999
;  Use as You wish, modify if You want
;  For more cool, unique gags and gadgets for GameBoy visit:
;  http://www.jps.net/paninaro
;  http://www.anthrox.com
;*****************************************************************************************

SECTION "Main Mem", BSS[$C000]

linecounter: ds 1
sineoffset: ds 2
priority: ds 1
flicker: ds 1
glowoffset: ds 1
logooffset: ds 1




         



    ;******************************************************************
    ;                            Header Equates
    ;******************************************************************
;Cartridge type:
; 0 - Rom only      3 - ROM+MBC1+RAM+Battery  1 - ROM+MBC1      5 - ROM+MBC2
; 2 - ROM+MBC1+RAM  6 - ROM+MBC2+Battery
CARTTYPE  EQU 2
;Rom Size:
; 0 - 256kBit =  32kB =  2 banks ; 1 - 512kBit =  64kB =  4 banks
; 2 -   1MBit = 128kB =  8 banks ; 3 -   2MBit = 256kB = 16 banks
; 4 -   4MBit = 512kB = 32 banks
ROMSIZE   EQU 0
;Ram Size:
; 0 - None ; 1 -  16kBit =  2kB = 1 bank
; 3 - 256kBit = 32kB = 4 banks
RAMSIZE   EQU 3
;Manufacturer code (word):
MANUCODE  EQU 1
;Version Number (byte):
VERSION   EQU 1

;************************************************************************************
;     Do NOT modify unless you are familiar with header info *Modified for Game Boy Color*
;************************************************************************************

SECTION "Header",home[$0100]
  ;Jumper
  nop
  jp Start ; go to Start of program:
  ;Nintendo Scrolling Title Graphic
  DB $CE,$ED,$66,$66,$CC,$0D,$00,$0B,$03,$73,$00,$83,$00,$0C,$00,$0D
  DB $00,$08,$11,$1F,$88,$89,$00,$0E,$DC,$CC,$6E,$E6,$DD,$DD,$D9,$99
  DB $BB,$BB,$67,$63,$6E,$0E,$EC,$CC,$DD,$DC,$99,$9F,$BB,$B9,$33,$3E
  ;Title of the game
  db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128 ; Filled in by RGBFIX
  ;Not used
  db 0,0,0; last should be 3, won't work in emu unles it's 0
  ;Cartridge type:
  db CARTTYPE
  ;Rom Size:
  db ROMSIZE
  ;Ram Size:
  db RAMSIZE
  ;Manufacturer code:
  db 00, $33
  ;Version Number
  db VERSION
  ;Complement check
  db $0a
  ;Checksum
  dw 0





  ;*********************************************************************
  ;          This is the start of the program
  ;*********************************************************************

 ;SECTION "Main", home

Start:
  
  ld a, 01
  ldio [$4d], a
  stop             ;set to double speed!
  
  
  ld  SP,$FFF4          ; Init stack ptr
  
  di                    ; disable interrupts
  
  ld a, 0
  ldio  [$42],a         ; Y Scroll=0
  ldio  [$43],a         ; X Scroll = 0

  ld  a,%00000000       ; Turn off LCD Controller
  ldio [$40],a
 
  call copygfx          ; copy graphics to vram
  call copytiles        ; copy tiles to vram


  ld a,$80              ; set the palette at 0 and allow it to auto-increase
  ld [$68], a
   
  ld hl,$ff69
  
  ld [hl], $0          ; color 0
  ld [hl], $0

  
  ld [hl], $00          ; color 1
  ld [hl], $00
  
  ld [hl], $00          ; color 2
  ld [hl], $00
  
  ld [hl], $00          ; color 3
  ld [hl], $7f
   

 

  ld     a,%10010001    ; LCD Controller = On
                        ; BG Gfx = $8000
                        ; BG Tiles   = $9800
                        ; BG     = On
  ldio    [$40],a

     
  

   ld a,0
   ld [linecounter], a
   ld [sineoffset], a
   ld a,8
   ld [priority], a
   ld a, 0
   ld [flicker], a
   ld [glowoffset], a
   ld [logooffset], a

;========================================================================
;           This is the start of the raster routines
;========================================================================  
  
megaloop:
  ld a, 146              ; Vertical Blank occurs from 144-153
  ld b, a                ; store it in B
  
  
vblank:                  ; Read current Raster position
  ldio a,[$44]
  cp b                   ; compare it with  # stored in B 
  jp nz, vblank          ; keep testing until we get there

  ld a, [flicker]
  xor $40                 ; flips vertical positions of logo. Yes there is just one logo
  ld [flicker], a         ; in memory
  

  ld hl,logosine             ; get address of logo sine data
   
  ld a,[logooffset]          ; fetch offset
  ld e, a                    ; store it in E
  
  ld a, [flicker]      ; get flicker offset (just add it to the logo sine offset)
  add a, e             ; add E to A
  ld e, a              ; store it back into E
  ld d,0               ; clear D (must clear it after adding with A with E)
  
  
  add hl, de           ; add logo sine address with offset in de, leave in HL
   
  ld a, [hl]            ; get logo sine data
  ldio [$42], a         ; store it in Screen Y-scroll register

  ld a, [logooffset]
  inc a                 ; increase logo sine offset by one
  ld [logooffset], a


;===============================================================================
;                            This makes the logo glow
;===============================================================================

  
  ld hl,glowcolors     ; get address of glowing logo colors
   
  ld a,[glowoffset]    ; fetch offset
  and $3f              ; offset is only from 0 to 63
  scf                  ; set carry flag
  ccf                  ; clear carry flag
  rl a                 ; rotate left (glowoffset*2)
  ld e, a              ; store it in E
  ld d,0               ; clear D
  add hl, de           ; add glowing colors address with de, leave in HL
     
    
  ld a, $86                ;10000110   Auto increase, set to color 3 in palette 0
  ldio [$68], a
  ld a, [hl+]              ; get glowing color data
  ldio [$69], a
  ld a, [hl+]              ; get glowing color data
  ldio [$69], a
  
  ld a, [glowoffset]
  inc a                    ; increase address to get next color data
  ld [glowoffset], a
  
  
;=============================================================================
; Determine whether or not the colored raster bar is above or behind the logo   
; This is done simply by changing the tile data adress for each line.
; When visible it will show tiles from $9800, which has the logo
; When invisible it shows tiles at $9C00, which is filled with blank tiles
;=============================================================================

  ld a,[sineoffset]             ; get the sine data array offset
  cp 64                         ; is the color bar at the top
  jp z, colpriority
  cp 128+64                     ; is the color bar at the bottom
  jp z, bgpriority
  jp hopover                    ; if not either then just skip it

colpriority:
  ld a, 0
  ld [priority], a              ; invisible logo
  jp hopover
  
bgpriority:
  ld a, 8                        ; visible logo
  ld [priority], a    


;==========================================================================
;          Get raster bar position offset from sine table
;              And wait for that raster line to occur
;==========================================================================

hopover:
    ld hl,sinedata              ; get address of sine data
   
     ld a,[sineoffset]          ; fetch offset
     ld e, a                    ; store it in E
     ld d,0                     ; clear D
     add hl, de                 ; add sine data address with de, leave in HL
     
  ld a,[hl]			; get sine data
  ld b, a                       ; store it in B
  
  
firstline:                  ; Read current Raster position
  ldio a,[$44]
  cp b                      ; compare it with  # stored in B 
  jp nz, firstline          ; keep testing until we get there

;========================================================================================
;           Read color from color table and make a new color for 32 raster lines
;           And change tile data address at each line to hide logo when needed
;========================================================================================

  ld hl, rastercolors             ; Get address of Color Table
   
rasterread:
  ld a,$80
  ldio [$68], a             ; Palette at zero, allow auto-increase
   
  
  ld a, [hl+]                ; read color from table
  ldio [$69], a              ; store palette data
  ld a, [hl+]                ; read next color from table
  ldio [$69], a              ; store palette data
  
  ld a,[priority]            ; get color bar priority bit
  ld b, a                    ; store it in B
  ld     a,%10010001         ; load the LCDC Register data (LCD enabled, BG at 8000)
  or b                       ; or value with B. It flips between Tiles at $9800 and $9c00
  ldio    [$40],a            ; $9c00 is filled with blank tiles
  
  
  ldio a,[$44]              ; get current raster line
  ld b,a                    ; store it in B
  
endofline:                  ; wait one whole raster line
  ldio a,[$44]
  cp b                      ; If it's not the same line as B then it's the next line!
  jp z, endofline
  
 
  ld a, [linecounter]      ; We need to do 32 colored raster lines
  dec a                    ; decrease it
  ld [linecounter], a      ; store it back into the counter
  jp nz, rasterread        ; if it's not 0 then go back and do another colored line
  
  ld a,32                  ; # of lines to do 
  ld [linecounter], a      ; reset # of lines to do
  

;=======================================================================================
;     After the 32 lines are done, clean up by setting the background back to black
;     and by setting the tile data address back to $9800
;=======================================================================================
      
  ldio a,[$44]            ; get current raster line
  ld b,a                  ; store it in B
  
lastline:
  ldio a,[$44]            ; get current raster line
  cp b                    ; compare it with old raster line
  jp z, lastline          ; if it's not the same then loop and check again!
  
  ld     a,%10010001    ; LCD Controller = On
                        ; BG Chr = $8000
                        ; BG Bank= $9800
                        ; BG     = On
  ldio    [$40],a
  
  ld a,$80              ; Palette 0, auto increase
  ldio [$68], a
  ld a, $00             ; set color to black (for background)
  ldio [$69], a
  ld a, $00
  ldio [$69], a

 
    ld a,[sineoffset]  ; get sine data table offset
    inc a
    inc a              ; increase it twice (makes it faster)
    ld [sineoffset],a  ; store it back
       

; That's it! Just go back and wait for the Vertical Blank to happen to start all over again!
   

  jp megaloop        ; Do it all over again!
  

;***************************************************************
;*                          Copy Tile Gfx
;***************************************************************

copygfx:
 ld  bc,logogfx
 ld  hl,$8000
 ld  de, 1536         ;# of bytes to copy

copyloop:
 ldio  a,[$41]       ; Wait until you can write to vram
 and 2              
 jr  nz, copyloop   
     
 ld  a, [bc]
 ld  [hli],a
 inc  bc
 dec  de
 ld a,d
 cp $ff
 jp  nz,copyloop
 ret



;***************************************************************
;*                          Copy Tile Map
;***************************************************************

copytiles:
  ld  bc,logomap
  ld  hl,$9800
  ld  de, 1024          ;# of bytes to copy
copyloop2:

 ldio  a,[$41]          ; Wait until you can write to vram
 and  2              
 jr nz,copyloop2      
 ld  a,[bc]

 ld  [hli],a
 inc  bc
 dec  de
 ld  a,d
 cp  $ff
 jp  nz,copyloop2
 
  ld  hl,$9c00
  ld  de, 1024          ;# of bytes to erase
eraseloop:

 ldio  a,[$41]          ; Wait until you can write to vram
 and  2              
 jr nz,eraseloop      
 ld  a,$00

 ld  [hli],a
 inc  bc
 dec  de
 ld  a,d
 cp  $ff
 jp  nz,eraseloop
 ret


;=============================================================================
;                          The rest is all data
;=============================================================================
 
sinedata:
db  55,  56,  58,  59,  60,  62,  63,  64
db  66,  67,  68,  70,  71,  72,  73,  75
db  76,  77,  78,  80,  81,  82,  83,  84
db  85,  87,  88,  89,  90,  91,  92,  93
db  94,  95,  96,  97,  97,  98,  99,  100
db  101,  101,  102,  103,  103,  104,  105,  105
db  106,  106,  107,  107,  108,  108,  108,  109
db  109,  109,  109,  110,  110,  110,  110,  110
db  110,  110,  110,  110,  110,  110,  109,  109
db  109,  109,  108,  108,  108,  107,  107,  106
db  106,  105,  105,  104,  104,  103,  102,  102
db  101,  100,  99,  99,  98,  97,  96,  95
db  94,  93,  92,  91,  90,  89,  88,  87
db  86,  85,  84,  82,  81,  80,  79,  78
db  76,  75,  74,  73,  71,  70,  69,  67
db  66,  65,  64,  62,  61,  60,  58,  57
db  55,  54,  53,  51,  50,  49,  47,  46
db  45,  43,  42,  41,  40,  38,  37,  36
db  34,  33,  32,  31,  30,  28,  27,  26
db  25,  24,  23,  22,  21,  20,  19,  18
db  17,  16,  15,  14,  13,  12,  11,  10
db  10,  9,  8,  7,  7,  6,  6,  5
db  4,  4,  3,  3,  3,  2,  2,  1
db  1,  1,  1,  1,  0,  0,  0,  0
db  0,  0,  0,  0,  0,  0,  0,  1
db  1,  1,  1,  2,  2,  3,  3,  3
db  4,  4,  5,  6,  6,  7,  7,  8
db  9,  10,  10,  11,  12,  13,  14,  15
db  16,  16,  17,  18,  19,  20,  22,  23
db  24,  25,  26,  27,  28,  29,  31,  32
db  33,  34,  36,  37,  38,  39,  41,  42
db  43,  45,  46,  47,  49,  50,  51,  53


rastercolors:
DB $FF,$3D, $3F,$46, $7F,$4E, $BF,$56, $1F,$63, $5F,$6B, $9F,$73, $FF,$7F
DB $BF,$77, $9F,$73, $5F,$6B, $3F,$67, $1F,$63, $DF,$5A, $BF,$56, $9F,$52
DB $5F,$4A, $3F,$46, $FF,$3D, $DF,$39, $BF,$35, $7F,$2D, $5F,$29, $3F,$25
DB $FF,$1C, $DF,$18, $9F,$10, $7C,$C, $59,$8, $36,$4, $13,$0, $10,$0

glowcolors:
DB $0,$7C, $42,$70, $84,$68, $C6,$60, $8,$59, $4A,$51, $8C,$49, $CE,$41
DB $10,$3A, $52,$32, $94,$2A, $D6,$22, $18,$1B, $5A,$13, $9C,$B, $DE,$3
DB $9E,$7, $5E,$F, $1E,$17, $DE,$1E, $9E,$26, $5E,$2E, $1E,$36, $FE,$3D
DB $BE,$45, $7E,$4D, $3E,$55, $FE,$5C, $BE,$64, $7E,$6C, $3E,$74, $1F,$7C
DB $3F,$7C, $7F,$7C, $BF,$7C, $FF,$7C, $3F,$7D, $7F,$7D, $BF,$7D, $FF,$7D
DB $3F,$7E, $7F,$7E, $BF,$7E, $FF,$7E, $3F,$7F, $7F,$7F, $BF,$7F, $FF,$7F
DB $BD,$7F, $7B,$7F, $39,$7F, $F7,$7E, $B5,$7E, $73,$7E, $31,$7E, $EF,$7D
DB $AD,$7D, $6B,$7D, $29,$7D, $E7,$7C, $A5,$7C, $63,$7C, $21,$7C, $0,$7C

logosine:
db  0,  0, -1, -1, -1, -1, -2, -2
db -2, -3, -3, -3, -3, -3, -4, -4
db -4, -4, -4, -4, -4, -4, -4, -4
db -4, -4, -4, -4, -4, -4, -4, -4
db -4, -3, -3, -3, -3, -2, -2, -2
db -1, -1,  0,  0,  1,  1,  2,  2
db  3,  3,  4,  4,  5,  5,  6,  7
db  7,  8,  8,  9,  10,  10,  11,  11
db  12,  13,  13,  14,  14,  15,  15,  16
db  16,  17,  17,  18,  18,  19,  19,  19
db  20,  20,  20,  20,  21,  21,  21,  21
db  21,  21,  21,  21,  21,  21,  21,  21
db  21,  20,  20,  20,  19,  19,  19,  18
db  18,  17,  17,  16,  16,  15,  15,  14
db  13,  13,  12,  11,  10,  10,  9,  8
db  7,  6,  5,  5,  4,  3,  2,  1
db  0, -1, -2, -2, -3, -4, -5, -6
db -7, -8, -8, -9, -10, -11, -11, -12
db -13, -14, -14, -15, -15, -16, -17, -17
db -18, -18, -18, -19, -19, -20, -20, -20
db -20, -21, -21, -21, -21, -21, -21, -21
db -21, -21, -21, -21, -21, -20, -20, -20
db -20, -19, -19, -19, -18, -18, -17, -17
db -16, -16, -15, -15, -14, -14, -13, -13
db -12, -12, -11, -10, -10, -9, -9, -8
db -7, -7, -6, -6, -5, -4, -4, -3
db -3, -2, -2, -1, -1,  0,  0,  1
db  1,  1,  2,  2,  2,  3,  3,  3
db  3,  4,  4,  4,  4,  4,  4,  4
db  4,  4,  4,  4,  4,  4,  4,  4
db  4,  4,  4,  3,  3,  3,  3,  2
db  2,  2,  2,  1,  1,  1,  1,  0




;SECTION "Grafx",data[$4000]

logogfx:
 incbin "gfx.bin"

logomap:
 incbin "map2.bin"
 